/*
Copyright (c) 2006, Peter Arthur Loeb

The following licence has been copied from
http://www.opensource.org/licenses/mit-license.html

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/
package com.palserv.clock;

import java.awt.Color;
import java.awt.GridLayout;
import java.awt.Image;
import java.awt.image.BufferedImage;
import javax.swing.UIManager;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;
import java.util.Calendar;
import java.util.GregorianCalendar;

/**
 * Make a binary clock.
 * I have seen a physical version with LEDs and I loved it.
 * So I made one in Java.
 * It displays the time in six columns of four rows each.
 * Each column represents a decimal digit, 0-9.
 * Thus this display:
 * <pre>
 * +---+---+---+---+---+---+
 * |   |   |   | X |   |   |
 * +---+---+---+---+---+---+
 * |   | X |   |   |   | X |
 * +---+---+---+---+---+---+
 * |   |   |   |   | X | X |
 * +---+---+---+---+---+---+
 * |   | X | X | X | X | X |
 * +---+---+---+---+---+---+
 * </pre>
 * represents 06:19:37, or 6:19 and 37 seconds.
 */
public class BinaryClock {
/*******************************************************

	public methods

*********************************************************/

	/**
	 * Run the clock from the command line.
	 * All arguments are quietly ignored.
	 * Internally, this implementation
	 * looks like this:
	 * <pre>
	 *	   BinaryClock bc = new BinaryClock();
	 *	   bc.makeFrame();
	 *	   bc.runClock();
	 * </pre>
	 * This code could be used from another program.
	 * @param args ignored
	 */
	public static void main(String[] args) {
		BinaryClock bc = new BinaryClock();
		bc.getArgs(args);
		bc.makeFrame();
		bc.runClock();
	}

	/**
	 * Get the command line arguments.
	 * <ul>
	 * <li>-m Use &quot;military&quot; time</li>
	 * <li>-b Set background color (see
	 * {@link com.palserv.clock.BinaryClock#setColor setColor}
	 * for an explanation of the format of the color spec
	 * default is 206*206*206.
	 * </li>
	 * <li>-z Set foreground color for a &quot;zero&quot; (see
	 * {@link com.palserv.clock.BinaryClock#setColor setColor}
	 * for an explanation of the format of the color spec
	 * default is 0*0*206.
	 * </li>
	 * <li>-o Set foreground color for a &quot;one&quot; (see
	 * {@link com.palserv.clock.BinaryClock#setColor setColor}
	 * for an explanation of the format of the color spec
	 * default is 128*255*255.
	 * </li>
	 * <li>-l Set outline color (see
	 * {@link com.palserv.clock.BinaryClock#setColor setColor}
	 * for an explanation of the format of the color spec
	 * default is 0*0*0.
	 * </li>
	 * <li>-h Set height of one Bit - should probably be at least 40 - default is 80</li>
	 * <li>-w Set width of one Bit - should probably be at least 30 - default is 60</li>
	 * <li>-r Set radius of circle within Bit - should probably be at least 2 - default is 7</li>
	 * </ul>
	 * @param args the arguments as passed to main
	 */
	public void getArgs(String[] args) {
		for (int i = 0; i < args.length; i ++) {
			if (args[i].substring(0,1).equals("-")) {
				// option
				if (args[i].substring(1,2).equals("m")) {
					mUseMilitary = true;
				} else if (args[i].substring(1,2).equals("b")) {
					setColor(Bit.BACKGROUND, args[i].substring(2));
				} else if (args[i].substring(1,2).equals("o")) {
					setColor(Bit.FOREGROUND_ONE, args[i].substring(2));
				} else if (args[i].substring(1,2).equals("z")) {
					setColor(Bit.FOREGROUND_ZERO, args[i].substring(2));
				} else if (args[i].substring(1,2).equals("l")) {
					setColor(Bit.OUTLINE, args[i].substring(2));
				} else if (args[i].substring(1,2).equals("h")) {
					int x = getIntFromString(args[i].substring(2));
					if (x != -1) {
						mHeight = x;
					}
				} else if (args[i].substring(1,2).equals("w")) {
					int x = getIntFromString(args[i].substring(2));
					if (x != -1) {
						mWidth = x;
					}
				} else if (args[i].substring(1,2).equals("r")) {
					int x = getIntFromString(args[i].substring(2));
					if (x != -1) {
						mRadius = x;
					}
				} else {
					spt("Unknown option ignored: "+args[i]);
				}
			} else {
				// non-option
				spt("Unknown argument ignored: "+args[i]);
			}
		}
	}

	/**
	 * Run the clock.
	 * Requires certain initialization,
	 * in particular, makeContentPane
	 * should be invoked before
	 * this method.
	 */
	public void runClock() {
		mCal = new GregorianCalendar();
		while(true) {
			try {
				Thread.sleep(100);	// wake up 10 times a second
			} catch(InterruptedException e) {
				// ignore
			}
			// give the calendar the current time
			mCal.setTimeInMillis(System.currentTimeMillis());
			// go set the clock
			setClock(mCal.get(Calendar.HOUR_OF_DAY), mCal.get(Calendar.MINUTE), mCal.get(Calendar.SECOND));
		}
	}

	/**
	 * Create a frame so the GUI can run in
	 * a separate window.
	 * Sets the content pane of the JFrame to
	 * the return from makeContentPane.
	 */
	public void makeFrame() {
		JFrame jf = new JFrame("Binary Clock");
		jf.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		jf.setContentPane(makeContentPane());
		jf.setIconImage(makeIcon());
		jf.setSize(mWidth * mCols - mCols - 4, mHeight * mRows - mRows - 2);
		jf.setVisible(true);
	}

	/**
	 * Create the clock in a JPanel.
	 * @return the clock.
	 */
	public JPanel makeContentPane() {
		try {
			UIManager.setLookAndFeel(
				UIManager.getSystemLookAndFeelClassName());
		} catch(Exception e) {
			spt("Unable to set system laf: "+e.getMessage());
		}
		JPanel pan = new JPanel();
		pan.setBorder(new EmptyBorder(0,0,0,0));
		pan.setLayout(new GridLayout(mRows, mCols));
		mBits = new Bit[mRows][mCols];
		for (int r = 0; r < mRows; r ++) {
			for (int c = 0; c < mCols; c ++) {
				mBits[r][c] = new Bit(mWidth, mHeight, mRadius);
				setColors(mBits[r][c]);
				pan.add(mBits[r][c]);
			}
		}
		// turn off selected mBits
		mBits[0][0].setShowCircle(false);
		mBits[0][2].setShowCircle(false);
		mBits[0][4].setShowCircle(false);
		mBits[1][0].setShowCircle(false);
		if (!mUseMilitary) {
			mBits[2][0].setShowCircle(false);
		}
		mTime = new int[mCols / 2];
		return pan;
	}

/*******************************************************

	private methods

*********************************************************/

	/**
	 * Create an icon for the application.
	 * It will create an image using the
	 * same colors as the display.
	 * This eliminates the need for
	 * for an image file for the
	 * application icon.
	 * @return the icon
	 */
	private Image makeIcon() {
		int h = 40; // height
		int w = 40; // width
		int r = 6;	// "radius" of circle
		int backRGB = colors[Bit.BACKGROUND].getRGB();
		int foreZeroRGB = colors[Bit.FOREGROUND_ZERO].getRGB();
		int foreOneRGB = colors[Bit.FOREGROUND_ONE].getRGB();
		int outlineRGB = colors[Bit.OUTLINE].getRGB();
		BufferedImage bi = new BufferedImage(w, h, BufferedImage.TYPE_INT_RGB);
		// do background
		for (int i = 0; i < h; i ++) {
			for (int j = 0; j < w; j ++) {
				bi.setRGB(j, i, backRGB);
			}
		}
		// fill "circle" - we will use a rectangle for simplicity
		int xCent = w / 2;		// x center
		int yCent = h / 2;		// y center
		int xBeg = xCent - r;	// x begin
		int xEnd = xCent + r;	// x end
		int yBeg = yCent - r;	// y begin
		int yEnd = yCent + r;	// y end
		for (int i = xBeg; i < xEnd; i ++) {
			for (int j = yBeg; j < yEnd; j ++) {
					bi.setRGB(i, j, foreOneRGB);
			}
		}
		return bi;
	}

	/**
	 * Set a color (by index) to a specific RGB value.
	 * The index must be in the range 0-3 and should
	 * be coded using one of the Bit constants:
	 * Bit.BACKGROUND, Bit.FOREGROUND_ZERO,
	 * Bit.FOREGROUND_ONE, and Bit.OUTLINE.
	 * The color specification is a String
	 * in the form:
	 * <pre>
	 * red*green*blue
	 * </pre>
	 * where red, green, and blue are ints in
	 * the range of 0-255 and the asterisks
	 * must be coded as shown.
	 * <p>
	 * As an example, to code red for the background
	 * use
	 * <pre>
	 * setColor(Bit.BACKGROUND, "255*0*0");
	 * </pre>
	 * @param ndx the index (for the colors array)
	 * @param colorSpec the color specification in the form r*g*b
	 */
	private void setColor(int ndx, String colorSpec) {
		if (ndx < 0 || ndx >= colors.length) {
			spt("Invalid index for setColor: "+ndx);
			return;
		}
		String[] vals = colorSpec.split("\\*");
		if (vals.length != 3) {
			spt("Invalid color spec: "+colorSpec+" - wrong number of colors");
			return;
		}
		int[] rgb = new int[3];
		for (int i = 0; i < 3; i ++) {
			rgb[i] = getIntFromString(vals[i]);
			if (rgb[i] == -1) {
				spt("Invalid color spec: "+colorSpec+" - not an int: "+vals[i]);
				return;
			}
			if (rgb[i] < 0 || rgb[i] > 255) {
				spt("Invalid color spec: "+colorSpec+" - int out of range: "+vals[i]);
				return;
			}
		}
		colors[ndx] = new Color(rgb[0], rgb[1], rgb[2]);
	}

	/**
	 * Get an int from a string representation of an int.
	 * @param s string representation of int
	 * @return the int or -1 if s is not an int
	 */
	private int getIntFromString(String s) {
		int x = -1;
		try {
			x = Integer.parseInt(s);
		} catch(NumberFormatException e) {
			x = -1;
		}
		return x;
	}

	/**
	 * Set the colors of a Bit.
	 * @param b the Bit to set.
	 */
	private void setColors(Bit b) {
		for (int i = 0; i < colors.length; i ++) {
			b.setColor(i, colors[i]);
		}
	}

	/**
	 * Set the time to the given BTime object.
	 * @param hour the current hour
	 * @param minute the current minute
	 * @param second the current second
	 */
	public void setClock(int hour, int minute, int second) {
		if (!mUseMilitary) {
			// display "military" time in a civilian manner (13-23 ==> 1-11, as in pm)
			if (hour > 12) {
				hour -= 12;
			}
			// display zero as twelve
			if (hour == 0) {
				hour = 12;
			}
		}
		// if the time hasn't changed, get out
		if (hour == mTime[0] && minute == mTime[1] && second == mTime[2]) {
			return;
		}
		mTime[0] = hour;
		mTime[1] = minute;
		mTime[2] = second;
		// for each (hours, minutes and seconds)
		for (int c = 0; c < mTime.length; c ++) {
			int t = mTime[c];		// get the number
			int tens = t / 10;		// get the "tens" column
			int ones = t % 10;		// get the "ones" column
			setBit(c * 2, tens);	// set the "tens" column
			setBit(c * 2 + 1, ones);	// set the "ones" column
		}
	}

	/**
	 * Set one &quot;bit&quot; - one column (four &quot;dots&quot;).
	 * @param c the column (0-5)
	 * @param t the value for the column (0-9)
	 */
	private void setBit(int c, int t) {
		if (t >= pow2(mRows)) {
			throw new Error("setBit - number too large: "+t);
		}
		int x = t;	// x = value
		// for (int j = mRows - 1; j >= 0; j --) {
		for (int i = 0; i < mRows; i ++) {
			int j = mRows - i - 1;	// set j
			int r = pow2(j);		// r = 2 ** j
			int y = x / r;			// y = quotient
			x = x % r;				// x = remainder
			mBits[i][c].setValue(y); // set bit from quotient (0 or 1)
		}
	}

	/**
	 * Calculate a power of two.
	 * @param p the power of two
	 * @return two to the power of the argument p
	 */
	private int pow2(int p) {
		int r = 1;
		for (int i = 0; i < p; i ++) {
			r *= 2;
		}
		return r;
	}

	/**
	 * Convenience method for writing a string to the console.
	 * @param s the string to write
	 */
	private void spt(String s) {
		System.out.println(s);
	}

/*******************************************************************************

	Private member variables

*******************************************************************************/

	/* use "military" time */
	private boolean mUseMilitary = false;

	/* number of rows */
	private static final int mRows = 4;

	/* number of columns */
	private static final int mCols = 6;

	/* width of cell */
	private int mWidth = 60;

	/* height of cell */
	private int mHeight = 80;

	/* radius of circle */
	private int mRadius = 7;

	/* colors to use */
	private Color[] colors = {
		new Color(206, 206, 206),
		new Color(0,0,206),
		new Color(128, 255, 255),
		Color.black};

	/* array of Bit objects */
	private Bit[][] mBits;

	/* array of three ints: hours, minutes and seconds */
	private int[] mTime;

	/* the calendar to use to get the time */
	private Calendar mCal;
}
