/*
Copyright (c) 2006, Peter Arthur Loeb

The following licence has been copied from
http://www.opensource.org/licenses/mit-license.html

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/
package com.palserv.clock;

import java.util.List;
import java.util.ArrayList;
import javax.swing.JApplet;

/**
 * Run a BinaryClock in an applet.
 * This applet allows quite a few parameters:
 * <ul>
 * <li>background - the background color</li>
 * <li>foreground_zero - color inside the circle for a &quot;zero&quot;</li>
 * <li>foreground_one - color inside the circle for a &quot;one&quot;</li>
 * <li>outline - color of the line around the circle</li>
 * <li>height - the height (in pixels) of one Bit</li>
 * <li>width - the width (in pixels) of one Bit</li>
 * <li>radius - the radius (in pixels) of the circle</li>
 * </ul>
 * For the colors, the value must be in the form:
 * <pre>
 * red*green*blue
 * </pre>
 * Where red, green and blue are int value from 0-255
 * and the asterisks are the delimiters.
 * As an example, to code a red background:
 * <pre>
 * &lt;param name=&quot;background&quot; value=&quot;255*0*0&quot;&gt;
 * </pre>
 * <p>
 * For the numbers, there are some considerations:
 * <p>
 * First, the size of the applet (which is controlled
 * by the height and width attributes of the applet tag)
 * should be related to the size of the Bits.
 * There are four rows of six columns of Bit objects.
 * If the applet size is too large, you may see some
 * other color peeking out from around the edges of the
 * Bit objects.
 * If the applet size is too small, you may see some
 * compression of the Bit images.
 * You may have to play with the sizes to get the
 * effect you want.
 */
public class BinaryClockApplet extends JApplet {
/***************************************************************

	public methods

****************************************************************/

	/**
	 * Initialize binary clock
	 */
	public void init() {
		mClock = new BinaryClock();
		getParms();
		setContentPane(mClock.makeContentPane());
		setVisible(true);
	}

	/**
	 * Start the clock
	 */
	public void start() {
		mClock.runClock();
	}

/***************************************************************

	private methods

****************************************************************/

	/**
	 * get the parameters from the applet tag.
	 */
	private void getParms() {
		List<String> parmList = new ArrayList<String>();
		for (int i = 0; i < parms.length; i ++) {
			String p = getParameter(parms[i]);
			if (p != null) {
				parmList.add("-"+options[i]+p);
			}
		}
		mClock.getArgs(parmList.toArray(new String[0]));
	}

/***************************************************************

	private member variables

****************************************************************/

	/**
	 * The parameters which are allowed on the applet param tag.
	 * The param element is a child of applet and has attributes
	 * name and value (with the obvious meanings).
	 * The first four control the colors.
	 * The last three control the size of each Bit
	 * and the little circles.
	 * The order must correspond to the order of the options array.
	 */
	private String[] parms = {
		"background",
		"foreground_zero",
		"foreground_one",
		"outline",
		"bit_height",
		"bit_width",
		"radius"
	};

	/**
	 * The (command line) options which correspond to the parms array.
	 * These are used to set options in BinaryClock.
	 * The order must correspond to the order of the parms array.
	 */
	private String[] options = {"b", "z", "o", "l", "h", "w", "r"};

	/* the clock */
	private BinaryClock mClock;

	/* for serialization */
	private static final long serialVersionUID = 234567l;
}
