/*
Copyright (c) 2006, Peter Arthur Loeb

The following licence has been copied from
http://www.opensource.org/licenses/mit-license.html

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/
package com.palserv.clock;

import java.awt.Color;
import java.awt.Graphics;
import javax.swing.JLabel;

/**
 * GUI representation of a binary digit.
 * It written as part of a binary clock,
 * but it could be used in any GUI which
 * wants a one or more binary digits
 * represented visually.
 */
public class Bit extends JLabel {
/*******************************************************

	public constructors

*******************************************************/

	/**
	 * Create a Bit object with specified sizes.
	 * @param w the width of the Bit object
	 * @param h the height of the Bit object
	 * @param r the radius of the circle
	 */
	public Bit(int w, int h, int r) {
		mWidth = w;
		mHeight = h;
		mRadius = r;
		setup();
	}

	/**
	 * Create a Bit object with default sizes.
	 */
	public Bit() {
		setup();
	}

/*******************************************************

	public methods

*******************************************************/

	/**
	 * Set the value of the Bit object.
	 * Only accepts int values of 0 and 1.
	 * @param v must be 0 or 1
	 */
	public void setValue(int v) {
		if (v == mValue) {
			return;
		}
		if (v > 1) {
			throw new Error("Bit.setValue: v too large: "+v);
		}
		if (v < 0) {
			throw new Error("Bit.setValue: v too small: "+v);
		}
		mValue = v;
		setLabel();
	}

	/**
	 * Tell this Bit object whether or not to show the circle.
	 * If this method is not called, the default is true.
	 * This is useful in a grid of bits where some are never used.
	 * One example of this is the BinaryClock - the top-most
	 * bits for minutes and seconds are never used because
	 * the value never goes above five.
	 * @param s whether or not to show the circle
	 */
	public void setShowCircle(boolean s) {
		mShowCircle = s;
	}

	/**
	 * Override the JComponent paint method.
	 * There are four colors used in this
	 * class.
	 * They are kept in an array of Color
	 * objects.
	 * The array is indexed by ints which
	 * are constants for this method.
	 * @param g the Graphics object to pain
	 */
	public void paint(Graphics g) {
		g.setColor(colors[BACKGROUND]);
		g.fillRect(0, 0, mWidth, mHeight);
		if (!mShowCircle) {
			return;
		}
		if (mValue == 0) {
			g.setColor(colors[FOREGROUND_ZERO]);
		} else if (mValue == 1) {
			g.setColor(colors[FOREGROUND_ONE]);
		} else {
			throw new Error("Bit.paint: invalid value: "+mValue);
		}
		g.fillOval(mULCx, mULCy, mDiameter + 1, mDiameter + 1);
		g.setColor(colors[OUTLINE]);
		g.drawOval(mULCx, mULCy, mDiameter, mDiameter);
	}

	/**
	 * Allow colors to be set from outside.
	 * @param n the index into the array of colors (0-3)
	 * @param c the color to use
	 */
	public void setColor(int n, Color c) {
		if (n < 0 || n >= colors.length) {
			throw new Error("Bit.setColor: invalid index: "+n);
		}
		colors[n] = c;
	}

	/**
	 * Set the height of this Bit object.
	 * @param h the height
	 */
	public void setHeight(int h) {
		mHeight = h;
	}

	/**
	 * Set the width of this Bit object.
	 * @param w the width
	 */
	public void setWidth(int w) {
		mWidth = w;
	}

	/**
	 * Set the radius of the circle within this Bit object.
	 * @param r the radius
	 */
	public void setRadius(int r) {
		mRadius = r;
	}

/*******************************************************

	private methods

*******************************************************/

	/**
	 * Do some initialization stuff.
	 */
	private void setup() {
		mValue = 0; // initialize the value
		mULCy = (mHeight / 2) - mRadius;
		mULCx = (mWidth / 2) - mRadius;
		mDiameter = mRadius * 2;
		setSize(mWidth, mHeight);
	}

	/**
	 * Set the visible display from the internal state.
	 */
	private void setLabel() {
		/* use the internal mValue to choose which icon to display */
		repaint();

	}

/*******************************************************************************

	Public class variables (constants)

*******************************************************************************/

	/**
	 * Index int the array of colors.
	 * This is the background color.
	 * It includes the area outside the circle.
	 * If the circle is not shown, then
	 * the color at this index
	 * covers the whole Bit object.
	 */
	public static final int BACKGROUND = 0;

	/**
	 * Index int the array of colors.
	 * This is the color for the inside of the circle
	 * when the bit is set to zero.
	 * It represents the &quot;off&quot; state.
	 */
	public static final int FOREGROUND_ZERO = 1;

	/**
	 * Index int the array of colors.
	 * This is the color for the inside of the circle
	 * when the bit is set to one.
	 * It represents the &quot;on&quot; state.
	 */
	public static final int FOREGROUND_ONE = 2;

	/**
	 * Index int the array of colors.
	 * This is the color for the outline of
	 * the circle in the middle of the Bit object.
	 */
	public static final int OUTLINE = 3;

/*******************************************************************************

	Private member variables

*******************************************************************************/

	/* the value of the Bit (must be 0 or 1) */
	private int mValue;

	/* show a circle ? */
	private boolean mShowCircle = true;

	/* the height of the Bit */
	private int mHeight = 40;

	/* the width of the Bit */
	private int mWidth = 40;

	/* the radius of the circle */
	private int mRadius = 4;

	/**
	 * Calculated in setup, used in paint.
	 * This is the &quot;x&quot; value of
	 * the Upper Left Corner (ULC) of the
	 * bounding rectangle of the circle.
	 */
	private int mULCx;

	/**
	 * Calculated in setup, used in paint.
	 * This is the &quot;y&quot; value of
	 * the Upper Left Corner (ULC) of the
	 * bounding rectangle of the circle.
	 */
	private int mULCy;

	/**
	 * Calculated in setup, used in paint.
	 * This is the diameter of the circle.
	 */
	private int mDiameter;

	/**
	 * The array of colors.
	 * There should be exactly four.
	 * This array is indexed with constants defined in this class.
	 * The values of the array can be set from outside
	 * using the setColors method.
	 */
	private Color[] colors = {Color.gray, Color.DARK_GRAY, Color.cyan, Color.black};

	/* for serialization */
	private static final long serialVersionUID = 123456l;
}
